@@ -35,9 +35,10 @@ TYPE_FIELD = {
 }
 
 data_dir = "data"
-DATA = {}
-DATA["regex"] = regexes
-DATA["description"] = descriptions
+DATA = {
+    "regex": regexes,
+    "description": descriptions,
+}
 
 CONFIG = {}
 
@@ -112,6 +113,14 @@ def create_dn_list(title, dn, index=None):
     return result
 
 
+def get_all_uniq_keys(data):
+    keys = {}
+    for item in data:
+        for k, v in item.items():
+            keys[k] = v
+    return keys
+            
+
 def one_to_one_looper(data, title=None, dn="", has_context_data=False):
     type_ = str_type(data)
     type_ = TYPE_TRANSLATE.get(type_, type_)
@@ -147,12 +156,16 @@ def one_to_one_looper(data, title=None, dn="", has_context_data=False):
             if "dict" in types_found:
                 nested_result = {}
                 result["nested"] = nested_result
-                for k, v in data[0].items():
+                
+                for k, v in get_all_uniq_keys(data).items():
                     nested_result[k] = one_to_one_looper(v, title=k, dn=create_dn(dn, k), has_context_data=True)
             else:
                 type_found = list(types_found.keys())[0]
                 result["item"] = one_to_one_looper(data[0], dn=create_dn_list(result.get("title"), dn))
 
+        elif len(types_found) == 0:
+            result["item"] = one_to_one_looper("", dn=create_dn_list(result.get("title"), dn))
+
         else:
             result["items"] = []
             for type_found, index in types_found.items():
